/*
  bclass.h
  
  Definitions for some basically classes

  Copyright (c) 1995-96 by Dragos Acostachioaie & Adrian Teohar
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _BCLASS_H_
#define _BCLASS_H_

#define max(a, b) ((a > b) ? a : b)
#define min(a, b) ((a < b) ? a : b)

class XPoint {
 public:
  unsigned x, y;
  XPoint();
  XPoint(unsigned _x, unsigned _y);
  void assign(unsigned _x, unsigned _y);
  XPoint& operator += (XPoint p);
  XPoint& operator -= (XPoint p);
  XPoint operator + (XPoint p);
  XPoint operator - (XPoint p);
  XPoint& operator = (XPoint p);
  int operator == (XPoint p);
  int operator != (XPoint p);
  int operator < (XPoint p);
  int operator > (XPoint p);
  int operator <= (XPoint p);
  int operator >= (XPoint p);
};

class XRect {
 public:
  XPoint a, b;
  XRect();
  XRect(unsigned XA, unsigned YA, unsigned XB, unsigned YB);
  XRect(XPoint _a, XPoint _b);
  void assign(XPoint _a, XPoint _b);
  void assign(unsigned XA, unsigned YA, unsigned XB, unsigned YB);
  void amove(int aDeltaX, int aDeltaY);
  void grow(int aDeltaX, int aDeltaY);
  void intersect(XRect r);
  void Union(XRect r);
  int contains(XPoint p);
  int operator == (XRect r);
  int operator != (XRect r);
};

inline void XPoint::assign(unsigned _x, unsigned _y)
{
  x = _x;
  y = _y;
}

inline XPoint::XPoint()
{
  assign(0, 0);
}

inline XPoint::XPoint(unsigned _x, unsigned _y)
{
  assign(_x, _y);
}


inline XPoint& XPoint::operator += (XPoint p)
{
  x += p.x;
  y += p.y;
  return *this;
}

inline XPoint& XPoint::operator -= (XPoint p)
{
  x -= p.x;
  y -= p.y;
  return *this;
}

inline XPoint XPoint::operator + (XPoint p)
{
  return XPoint(x + p.x, y + p.y);
}

inline XPoint XPoint::operator - (XPoint p)
{
  return XPoint(x - p.x, y - p.y);
}

inline XPoint& XPoint::operator = (XPoint p)
{
  x = p.x;
  y = p.y;
  return *this;
}

inline int XPoint::operator == (XPoint p)
{
  return x == p.x && y == p.y;
}

inline int XPoint::operator != (XPoint p)
{
  return !(*this == p);
}

inline int XPoint::operator < (XPoint p)
{
  return x < p.x && y < p.y;
}

inline int XPoint::operator > (XPoint p)
{
  return x > p.x && y > p.y;
}

inline int XPoint::operator <= (XPoint p)
{
  return x <= p.x && y <= p.y;
}

inline int XPoint::operator >= (XPoint p)
{
  return x >= p.x && y >= p.y;
}

inline void XRect::assign(XPoint _a, XPoint _b)
{
  a = _a;
  b = _b;
}

inline void XRect::assign(unsigned XA, unsigned YA, unsigned XB, unsigned YB)
{
  a.x = XA;
  a.y = YA;
  b.x = XB;
  b.y = YB;
}

inline XRect::XRect()
{
  assign(0, 0, 0, 0);
}

inline XRect::XRect(unsigned XA, unsigned YA, unsigned XB, unsigned YB)
{
  assign(XA, YA, XB, YB);
}

inline XRect::XRect(XPoint _a, XPoint _b)
{
  assign(_a, _b);
}

inline void XRect::amove(int aDeltaX, int aDeltaY)
{
  a.x = a.x + aDeltaX;
  a.y = a.y + aDeltaY;
  b.x = b.x + aDeltaX;
  b.y = b.y + aDeltaY;
}

inline void XRect::grow(int aDeltaX, int aDeltaY)
{
  b.x = b.x + aDeltaX;
  b.y = b.y + aDeltaY;
}

inline void XRect::intersect(XRect r)
{
  a.x = max(a.x, r.a.x);
  a.y = max(a.y, r.a.y);
  b.x = min(b.x, r.b.x);
  b.y = min(b.y, r.b.y);
}

inline void XRect::Union(XRect r)
{
  a.x = min(a.x, r.a.x);
  a.y = min(a.y, r.a.y);
  b.x = max(b.x, r.b.x);
  b.y = max(b.y, r.b.y);
}

inline int XRect::contains(XPoint p)
{
  return p.x >= a.x && p.x <= b.x && p.y >= a.y && p.y <= b.y;
}

inline int XRect::operator == (XRect r)
{
  return a == r.a && b == r.b;
}

inline int XRect::operator != (XRect r)
{
  return !(*this == r);
}

#endif
