/*
  event.h
  
  Definitions needed for event programming

  Copyright (c) 1995-96 by Dragos Acostachioaie & Adrian Teohar

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "bclass.h"
#include "types.h"

#ifndef _EVENT_H_
#define _EVENT_H_

// Event masks
#define EV_NOTHING   0x0000
#define EV_MOUSE     0x000F
#define EV_KEYBOARD  0x0010
#define EV_MESSAGE   0xFF00

// Event codes
#define EV_MOUSEDOWN 0x0001
#define EV_MOUSEUP   0x0002
#define EV_MOUSEMOVE 0x0004
#define EV_MOUSEAUTO 0x0008
#define EV_KEYDOWN   0x0010
#define EV_COMMAND   0x0100
#define EV_BROADCAST 0x0200

// Command codes
#define CM_QUIT      0x0001
#define CM_MENU      0x0002
#define CM_CLOSE     0x0004
#define CM_OK        0x000A
#define CM_CANCEL    0x000B

// Window messages
#define WM_CLOSE     0x0001
#define WM_OPEN      0x0002
#define WM_MOVE      0x0004

class XMessage {
 public:
  unsigned sid, rid;  /* Sender and Receiver's ID */
  unsigned msgClass;
  unsigned securityLevel;
  int message;
  char *msgtext;
  void *ptr;          /* so we can send pointers between objects */
};

struct XEvent {
  unsigned oid;
  int what;
  struct ev_mouse {
    int buttons;
    unsigned dbl;
    XPoint where;
  } evMouse;
  struct ev_keyboard {
    int keycode;
    char charcode;
    unsigned char scancode;
  } evKeyboard;
  XMessage evMessage;
};

class XMailer {
  friend XMessage;
  friend XEvent;
 public:
  XMailer() {};
  void getevent(XEvent *&ev, unsigned id);
  void delivermessage(XMessage msg);
};

extern XMailer *mailer;

/* Message classes */
#define MC_CLIENTTOSERVER 0x01
#define MC_CLIENTTOCLIENT 0x02
#define MC_SERVERTOCLIENT 0x03
#define MC_BROADCAST      0x04

/* Basically message texts */
#define MT_ANSWERREQUEST  0x0000
#define MT_REQUESTHONORED 0x0001
#define MT_RELEASEFOCUS   0x0002
#define MT_RESUMEFOCUS    0x0003

struct evQueue {
  XEvent ev;
  evQueue *head, *tail;
  evQueue *next;
};

extern evQueue *eventQueue;

void initEventQueue();
void putEvent(XEvent);
XEvent getEvent();
XEvent getNextEvent();
void clearEvent(XEvent *&);
int eventQueueIsEmpty();
void clearEventQueue();

int sendMessage(unsigned source, unsigned target, int msgclass, void *ptr);
int sendMessage(unsigned source, unsigned target, int msgclass, int message);

void initEvents();
void doneEvents();

#endif
