/*
  xobject.h
  
  Definitions for XObject and objects table

  Copyright (c) 1995-96 by Dragos Acostachioaie & Adrian Teohar

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _XOBJECT_H_
#define _XOBJECT_H_

#include "bclass.h"
#include "event.h"

/* XObject State Masks */
#define SM_VISIBLE       0x0001
#define SM_CURSORVISIBLE 0x0002
#define SM_FOCUSED       0x0004
#define SM_DRAGGING      0x0008
#define SM_DISABLED      0x0010
#define SM_EXPOSED       0x0020
#define SM_ACTIVE        0x0100

/* Object Classes */
#define OC_UNKNOWN       "UNKNOWN"
#define OC_XOBJECT       "XOBJECT"

extern unsigned newid; /* will be assigned to the next object created */

class XObject {
 protected:
  char *pal;
 public:
  unsigned id;  /* object's id */
  XPoint origin, size, cursor;
  XRect bounds; /* this are the *real* bounds, in absolute coordinates */
  XObject *server, *client;
  unsigned state;
  char *objclass;
  XObject(XRect _bounds);
  virtual ~XObject();
  virtual void calculatebounds();
  virtual void draw() {};
  virtual void getevent(XEvent *&ev);
  virtual int getstate(int mask);
  virtual void handleevent(XEvent *&) {};
  virtual void moveto(XPoint p);
  virtual void registerclient(XObject *&newclient);
  virtual void setstate(int mask, int set_bool);
  virtual char *getpalette();
  virtual void setpalette(char *_pal);
};

struct xobjectTable {
  unsigned oid;
  XObject *ptr;
  xobjectTable *next;
};

extern xobjectTable *objectTable;

void insertObject(XObject *&ptr);
void deleteObject(unsigned id);
int isObjectRegistered(unsigned id);
int isObjectRegistered(XObject *ptr);
XObject* getoptr(unsigned id);
unsigned getoid(XObject *ptr);

extern unsigned ispcconsole, isconsole, havecolors, cursorvisible;

void xinit();
void xdone();

#endif
