/*
  xtext.h

  Definitions for the most classes

  Copyright (c) 1995-96 by Dragos Acostachioaie & Adrian Teohar

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _XTEXT_H_
#define _XTEXT_H_

#include "xobject.h"

class XtWindow : public XObject {
 protected:
  chtype *buffer;
  char *title;
  char *frame;
  void frameupdate();
 public:
  XtWindow(XRect r, char *_title);
  virtual ~XtWindow();
  virtual void draw();
  virtual void close();
  virtual void handleevent(XEvent *&ev);
  virtual void moveto(XPoint p);
  virtual void setstate(int mask, int set_bool);
  virtual void setframe(char *_frame);
};

class XtBackground {
 private:
  chtype pattern;
  XRect bounds;
  char *pal;
 public:
  XtBackground(XRect r, chtype ch);
  virtual void draw();
  virtual void setpalette(char *_pal);
  virtual void setpattern(chtype _pattern);
};

/* Align styles */
#define AS_LEFT	     0x01
#define AS_CENTER    0x02
#define AS_RIGHT     0x03
#define AS_TOP 	     0x04
#define AS_BOTTOM    0x05
#define AS_MIDDLE    0x06

class XtStaticText : public XObject {
 private:
  char *text;
  char *pal;
  unsigned alignment;
 public:
  XtStaticText(XRect r, unsigned _alignment, char *_text);
  virtual void draw();
  virtual void setvalue(char *_text);
};

/* XtInputStringField control chars */
#define IS_ENCRYPTCONTROLCHAR '*'
#define IS_LEFTMARGIN         ACS_LARROW
#define IS_RIGHTMARGIN        ACS_RARROW
#define IS_ELEFTMARGIN        '['
#define IS_ERIGHTMARGIN       ']'
#define IS_EBACKGROUNDCHAR    '.'

class XtInputStringField : public XObject {
 private:
  unsigned fieldlength, stringlength, maxlen, CP, curX;
  char bchar;
  unsigned encrypt, insertmode, inputtext;
 public:
  char *defaultstring, *currstring;
  XtInputStringField(XRect r, char *_defaultstring, unsigned _maxlen, unsigned crypted);
  virtual ~XtInputStringField();
  virtual void draw();
  virtual void handleevent(XEvent *&ev);
  virtual void setstate(int mask, int set_bool);
};

class XtScrollBar : public XObject {
 private:
  bool horizontal;
  long currentno, maxno;
 public:
  XtScrollBar(XRect r, long totalno, bool _horizontal);
  virtual void draw();
  void setvalue(long itemno);
  void settotalno(long itemno);
  long gettotalno();
};  

/* Progress Bar directions */
#define PB_LEFTTORIGHT  0x01
#define PB_RIGHTTOLEFT  0x02
#define PB_DOWNTOUP     0x03
#define PB_UPTODOWN     0x04

class XtProgressBar : public XObject {
 private:
  long currentno, maxno;
  float percent;
  unsigned poz, len;
  unsigned direction;
 public:
  XtProgressBar(XRect r, long totalno, unsigned _direction);
  virtual void draw();
  float getpercent();
  void setvalue(long itemno);
  long getvalue();
  void settotalno(long itemno);
  long gettotalno();
};

#endif
