/*
  event.c

  Copyright (c) 1995-96 by Dragos Acostachioaie & Adrian Teohar

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "event.h"
#include "xobject.h"

XMailer *mailer;

void clearEvent(XEvent *&ev)
{
  ev->what = EV_NOTHING;
}

evQueue *eventQueue;

void initEventQueue()
{
  eventQueue = new evQueue;
  eventQueue->head = eventQueue->tail = 0;
}

void putEvent(XEvent ev)
{
  evQueue *temp;
  if(eventQueueIsEmpty())
    {
    eventQueue->head = new evQueue;
    eventQueue->head->ev = ev;
    eventQueue->head->next = 0;
    eventQueue->tail = eventQueue->head;
    }
  else
    {
    temp = new evQueue;
    temp->ev = ev;
    temp->next = 0;
    eventQueue->tail->next = temp;
    eventQueue->tail = temp;
    }
}

XEvent getEvent()
{
  evQueue *temp;
  XEvent atom;
  if(!eventQueueIsEmpty())
    {
    temp = eventQueue->head;
    eventQueue->head = eventQueue->head->next;
    atom = temp->ev;
    delete temp;
    return atom;
    }
  atom.what = EV_NOTHING;
  return atom;
}

XEvent getNextEvent()
{
  XEvent ev;
  if(!eventQueueIsEmpty())
    return eventQueue->head->ev;
  else
    {
    ev.what = EV_NOTHING;
    return ev;
    }
}

int eventQueueIsEmpty()
{
  return(eventQueue->head == 0);
}

void clearEventQueue()
{
  while(!eventQueueIsEmpty())
    getEvent();
  delete eventQueue;
}

int sendMessage(unsigned source, unsigned target, int msgclass, void *ptr)
{
/* if target does not exists, return 0 */
  XMessage msg;
  if(isObjectRegistered(target))
    {
    msg.sid = source;
    msg.rid = target;
    msg.msgClass = msgclass;
    msg.ptr = ptr;
    mailer->delivermessage(msg);
    return 1;
    }
  return 0;
}

int sendMessage(unsigned source, unsigned target, int msgclass, int message)
{
  XMessage msg;
  if(isObjectRegistered(target))
    {
    msg.sid = source;
    msg.rid = target;
    msg.msgClass = msgclass;
    msg.message = message;
    mailer->delivermessage(msg);
    return 1;
    }
  return 0;
}

void XMailer::getevent(XEvent *&ev, unsigned oid)
{
  if(eventQueue->head->ev.oid == oid)
    ev = &getEvent();
  else
    ev->what = EV_NOTHING;
}

void XMailer::delivermessage(XMessage msg)
{
  XEvent ev;
  xobjectTable *temp = objectTable;
  if(msg.msgClass == MC_BROADCAST)
    while(temp->next)
      {
      ev.evMessage = msg;
      ev.oid = temp->oid;
      putEvent(ev);
      temp = temp->next;
      }
  else
    {
    ev.evMessage = msg;
    ev.oid = msg.rid;
    putEvent(ev);
    }
}

void initEvents()
{
  initEventQueue();
  mailer = new XMailer;
}

void doneEvents()
{
  clearEventQueue();
  delete mailer;
}
