/*
  tscreen.c
  
  Copyright (c) 1995-96 by Dragos Acostachioaie & Adrian Teohar

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <ncurses.h>
#include <string.h>

#include "bclass.h"
#include "tscreen.h"
#include "xobject.h"

char fs_default[9] = "+-+||+-+";

void fillwindow(XRect r, int c)
{
  for(unsigned j = r.a.y; j <= r.b.y; j++)
    for(unsigned i = r.a.x; i <= r.b.x; i++)
      mvaddch(j, i, c);
}

void colorwindow(XRect r, int f, int b, unsigned long attr)
{
  init_pair(1, f, b);
  attrset(COLOR_PAIR(1) | attr);
  fillwindow(r, ' ');
}

void putwindow(XRect r, chtype *buff)
{
  unsigned t = 0;
  for(unsigned j = r.a.y; j <= r.b.y; j++)
    for(unsigned i = r.a.x; i <= r.b.x; i++)
      {
/* ... the following stuff just because ncurses' mvaddch() doesn't work :( */
      if(havecolors)        /* to avoid segmentation faults :-) */
        color_pairs[1] = ((buff[t] & A_COLOR) >> 2) + 0x07;
      else
        init_pair(1, (((buff[t] & A_COLOR) >> 2) + 0x07) / 16, (((buff[t] & A_COLOR) >> 2) + 0x07) % 16);
      stdscr->_attrs = COLOR_PAIR(1);
     #ifdef NCURSES_VERSION
/* ugly, but _line structure has been changed from ncurses 1.8.6 in 1.9.4 */
        stdscr->_line[j].text[i] = buff[t++];
        stdscr->_line[j].firstchar = r.a.x;
        stdscr->_line[j].lastchar = r.b.x;
       #else
        stdscr->_line[j][i] = buff[t++];
        stdscr->_firstchar[j] = r.a.x;
        stdscr->_lastchar[j] = r.b.x;
      #endif
      }
}

void getwindow(XRect r, chtype *&buff)
{
  unsigned t = 0;
  for(unsigned j = r.a.y; j <= r.b.y; j++)
    for(unsigned i = r.a.x; i <= r.b.x; i++)
      #ifdef NCURSES_VERSION      /* the same thing... :( */
        buff[t++] = stdscr->_line[j].text[i];
       #else
        buff[t++] = stdscr->_line[j][i];
      #endif
}

void _border(XRect r, char *frame)
{
  unsigned i, j;
  int isdefault = 0;
  #ifdef __i386__        /* this will allow to compile Xt on non-PC machines */
    if(ispcconsole)      /* may be we'll need (ispcconsole & isconsole) */
      attron(A_ALTCHARSET);
/* instead of A_PCCHARSET; ncurses 1.8.6 does not define it :( */
  #endif
  if(!strcmp(frame, fs_default))
    isdefault = 1;
  if(isdefault)
    mvaddch(r.a.y, r.a.x, ACS_ULCORNER);
   else
    mvaddch(r.a.y, r.a.x, (unsigned char)frame[0]);
  for(i = r.a.x + 1; i < r.b.x; i++)
    if(isdefault)
      {
      mvaddch(r.a.y, i, ACS_HLINE);
      mvaddch(r.b.y, i, ACS_HLINE);
      }
     else
      {
      mvaddch(r.a.y, i, (unsigned char)frame[1]);
      mvaddch(r.b.y, i, (unsigned char)frame[6]);
      }
  if(isdefault)
    mvaddch(r.a.y, r.b.x, ACS_URCORNER);
   else
    mvaddch(r.a.y, r.b.x, (unsigned char)frame[2]);
  for(j = r.a.y + 1; j < r.b.y; j++) 
    if(isdefault)
      {
      mvaddch(j, r.a.x, ACS_VLINE);
      mvaddch(j, r.b.x, ACS_VLINE);
      }
     else
      {
      mvaddch(j, r.a.x, (unsigned char)frame[3]);
      mvaddch(j, r.b.x, (unsigned char)frame[4]);
      }
  if(isdefault)
    {
    mvaddch(r.b.y, r.a.x, ACS_LLCORNER);
    mvaddch(r.b.y, r.b.x, ACS_LRCORNER);
    }
   else
    {
    mvaddch(r.b.y, r.a.x, (unsigned char)frame[5]);
    mvaddch(r.b.y, r.b.x, (unsigned char)frame[7]);
    }
}
